from Rare.Tabs.Browser import BrowserTab
from Rare.Tabs.GamesInstalled.InstalledList import GameListInstalled
from Rare.Tabs.GamesUninstalled.GameListUninstalled import GameListUninstalled
from Rare.Tabs.Settings.SettingsTab import SettingsTab
from Rare.Tabs.Update import UpdateTab
